/* 
   gencomp -- software for comparing transcript structures along a genome
   Copyright (C) 2010  Liliana Florea
   Report bugs or questions: florea@umiacs.umd.edu
*/

#include "libc.h"
#include "lib.h"
#include "compare.h"

static void exon_stats(ASV_t *,ASV_t *,int *,int *,int *,int *,int *,int,int);
static void intron_stats(ASV_t *,ASV_t *,int *,int *,int *,int *,int,int);
static int  compatible(ASV_t *,ASV_t *,int,int,int);
static int compatible_WEAK(ASV_t *,ASV_t *,int,int);
static int compatible_STRICT(ASV_t *,ASV_t *,int,int);

int transcript_structure_compare(ASV_t *ai,
                                 ASV_t *aj,
                                 int *startI,
                                 int *endI,
                                 int *OH3,
                                 int *OH5)
{
   Exon_t *xi, *xj;
   int     ni, nj;
   int     starti, startj, endi, endj;
   int     oh3, oh5;


   oh3 = 0; oh5 = 0;

   ni = 0; nj = 0;
   starti = -1; startj = -1;
   endi = -1; endj = -1;
   xi = ai->exons; xj = aj->exons;

   while (ni+1<ai->numExons || nj+1<aj->numExons) {
      if (ni+1<ai->numExons && nj+1<aj->numExons &&
          xi->to==xj->to && (xi+1)->from==(xj+1)->from) {

         if (!ni || !nj) {
            starti = ni; startj = nj;
         }

         if (!ni && nj && xi->from>=xj->from) {
            if (ai->strand==1) oh5 = 2; else oh3 = 2;
         } else if (!nj && ni && xj->from>=xi->from) {
            if (ai->strand==1) oh5 = 1; else oh3 = 1;
         }
         if (ni+2==ai->numExons && nj+2<aj->numExons && (xi+1)->to<=(xj+1)->to) {
            endi = ni+1; endj = nj+1;
            if (aj->strand==1) oh3 = 2; else oh5 = 2;
            break; 
         } else if (ni+2<ai->numExons && nj+2==aj->numExons && (xj+1)->to<=(xi+1)->to) {
            endi = ni+1; endj = nj+1;
            if (ai->strand==1) oh3 = 1; else oh5 = 1;
            break;
         }
         if (ni+2==ai->numExons && nj+2<aj->numExons && (xi+1)->to>=(xj+2)->from) {
            break;
         } else if (nj+2==aj->numExons && ni+2<ai->numExons && (xj+1)->to>=(xi+2)->from) {
            break;
         }
         endi = ni+1; endj = nj+1;
         ni++; nj++; xi++; xj++; 

      } else if (ai->numExons==1 && aj->numExons>1) {

         assert(!ni);
         if (overlaps(xi->from, xi->to, xj->from, xj->to)) {
             starti = ni; startj = nj;
             endi = ni; endj = nj;
             if (xi->from>=xj->from) {
                 if (aj->strand==1) oh5 = 2; else oh3 = 2;
             }
             if (xi->to<=xj->to) {
                 if (aj->strand==1) oh3 = 2; else oh5 = 2;
             }
         }
         break;

      } else if (aj->numExons==1 && ai->numExons>1) {

         assert(!nj);
         if (overlaps(xi->from, xi->to, xj->from, xj->to)) {
             starti = ni; startj = nj;
             endi = ni; endj = nj;
             if (xj->from>=xi->from) {
                 if (ai->strand==1) oh5 = 1; else oh3 = 1;
             }
             if (xj->to<=xi->to) {
                 if (ai->strand==1) oh3 = 1; else oh5 = 1;
             }
         }
         break;

      } else if (aj->numExons==1 && ai->numExons==1) {

        assert(!ni); assert(!nj);
        if (overlaps(xi->from, xi->to, xj->from, xj->to)) {
             starti = ni; startj = nj;
             endi = ni; endj = nj;
         }
         break;

      } else if (!ni && nj+1<aj->numExons && xj->to<xi->from) {

         nj++; xj++;

      } else if (!nj && ni+1<ai->numExons && xi->to<xj->from) {

         ni++; xi++;

      } else {

         break;

      }
   }

   *startI = starti; *endI   = endi; *OH3 = oh3; *OH5 = oh5;

   if (starti>=0 && startj>=0 && endi>=0 && endj>=0 &&
       (!starti || !startj) &&
       (endi==ai->numExons-1 || endj==aj->numExons-1)) {

      assert(endi-starti==endj-startj);

      return 1;
#if 0
      printf("%s %s %d %d:%d:%d %c %c %d %d\n",
             ai->asvId, aj->asvId, ai->graphId,
             ai->numExons, aj->numExons, endi-starti+1,
             (ai->strand==-1) ? '-' : '+',
             (ai->relori!=ai->strand || aj->relori!=aj->strand) ? '!':'*',
             oh5, oh3);
#endif
   }

   return 0;
}


int overlaps(int x, int y, int a, int b)
{
    return ((x<=a && a<=y) || (a<=x && x<=b));
}


int exon_overlap(ASV_t *x, ASV_t *y)
{
    int i, j;

    for (i=0; i<x->numExons; i++) {
       int b = x->exons[i].from;
       int e = x->exons[i].to;

       for (j=0; j<y->numExons; j++) 
          if (overlaps(b, e, y->exons[i].from, y->exons[j].to)) 
             return 1;
    }
    return 0;
}


int feature_compare(ASV_t *ai, ASV_t *aj, CompareStats_t *es, CompareStats_t *et, int Vi, int Vm, int mode)
{
    int   i, j, l, L, U, st;
    int   genLo, genHi;
    char *vi, *vj;

    int   exactXi=0, partialXi=0, extendedXi=0, weakXi=0, nomatchXi=0;
    int   exactXj=0, partialXj=0, extendedXj=0, weakXj=0, nomatchXj=0;
    int   exactIi=0, partialIi=0, weakIi=0, nomatchIi=0;
    int   exactIj=0, partialIj=0, weakIj=0, nomatchIj=0;
    int   fp=0, fn=0, tp=0, tn=0;

    int   compatibleXi, compatibleXj, compatibleIi, compatibleIj;
    int   compatibleBi, compatibleBj, compatibleBn;


    genLo = min(ai->b, aj->b);
    genHi = max(ai->e, aj->e);

    vi = (char *)malloc((genHi-genLo+1)*sizeof(char));
    vj = (char *)malloc((genHi-genLo+1)*sizeof(char));
    memset(vi, 0, (genHi-genLo+1)*sizeof(char)); vi -= genLo;
    memset(vj, 0, (genHi-genLo+1)*sizeof(char)); vj -= genLo;

    for (i=0; i<ai->numExons; i++)
       for (l=ai->exons[i].from; l<=ai->exons[i].to; l++)
          vi[l] = 1;

    for (j=0; j<aj->numExons; j++)
       for (l=aj->exons[j].from; l<=aj->exons[j].to; l++)
          vj[l] = 1;

    /* base pair statistics: ai is the 'gold reference' */

    for (i=genLo; i<=genHi; i++) {
       if (vi[i] && vj[i]) tp++;
       else if (vi[i] && !vj[i]) fn++;
       else if (!vi[i] && vj[i]) fp++;
       else tn++;
    }

    /* exon pairing statistics */
    exon_stats(ai, aj, &exactXi, &extendedXi, &partialXi, &weakXi, &nomatchXi, Vm, Vi);
    exon_stats(aj, ai, &exactXj, &extendedXj, &partialXj, &weakXj, &nomatchXj, Vm, Vi);

    /* intron pairing statistics */
    intron_stats(ai, aj, &exactIi, &partialIi, &weakIi, &nomatchIi, Vm, Vi);
    intron_stats(aj, ai, &exactIj, &partialIj, &weakIj, &nomatchIj, Vm, Vi);

    compatibleXi = compatibleXj = 0;
    compatibleIi = compatibleIj = 0;
    compatibleBi = compatibleBj = compatibleBn = 0;

    st = -1;

    /* compatible(i) will reduce FN; compatible(j) will reduce FP */
    if (ai->type==TXPT_TYPE && aj->type==TXPT_TYPE && (st=compatible(ai,aj,Vi,Vm,mode))!=INCOMPATIBLE) {

        /* exon level */
        for (i=0;
             i<ai->numExons-1 && ai->exons[i].to<aj->exons[0].from+Vi;
             i++, compatibleXi++);

        for (j=0;
             j<aj->numExons-1 && aj->exons[j].to<ai->exons[0].from+Vi;
             j++, compatibleXj++);

        for (i=ai->numExons-1;
             i>=0 && ai->exons[i].from>aj->exons[aj->numExons-1].to-Vi;
             i--, compatibleXi++);

        for (j=aj->numExons-1;
             j>=0 && aj->exons[j].from>ai->exons[ai->numExons-1].to-Vi;
             j--, compatibleXj++);

        /* intron level -- same numbers as exon level */

        /* bp level */
        L = max(ai->exons[0].from, aj->exons[0].from);
        U = min(ai->exons[ai->numExons-1].to, aj->exons[aj->numExons-1].to);

        for (i=genLo; i<L; i++) {
            if (!vi[i] && !vj[i]) compatibleBn++;
            else if (vi[i] && !vj[i]) compatibleBi++;
            else if (!vi[i] && vj[i]) compatibleBj++;
        }
        for (i=genHi; i>U; i--) {
            if (!vi[i] && !vj[i]) compatibleBn++;
            else if (vi[i] && !vj[i]) compatibleBi++;
            else if (!vi[i] && vj[i]) compatibleBj++;
        }
    }

    free(vi+genLo); free(vj+genLo);

    es->numExonExactMatch      = exactXi;
    es->numExonExtendedMatch   = extendedXi;
    es->numExonPartialMatch    = partialXi;
    es->numExonWeakMatch       = weakXi;
    es->numExonNoMatch         = nomatchXi;

    es->numBpTP                = tp;
    es->numBpFP                = fp;
    es->numBpTN                = tn;
    es->numBpFN                = fn;

    es->numIntronExactMatch    = exactIi;
    es->numIntronPartialMatch  = partialIi;
    es->numIntronWeakMatch     = weakIi;
    es->numIntronNoMatch       = nomatchIi;

    et->numExonExactMatch      = exactXj;
    et->numExonExtendedMatch   = extendedXj;
    et->numExonPartialMatch    = partialXj;
    et->numExonWeakMatch       = weakXj;
    et->numExonNoMatch         = nomatchXj;

    et->numBpTP                = tp;
    et->numBpFP                = fp;
    et->numBpTN                = tn;
    et->numBpFN                = fn;

    et->numIntronExactMatch    = exactIj;
    et->numIntronPartialMatch  = partialIj;
    et->numIntronWeakMatch     = weakIj;
    et->numIntronNoMatch       = nomatchIj;


    printf(">%s %s %d %d %d.%d %d.%d %d | %d %d | X %d %d %d %d %d %d %d %d %d %d | %d %d | I %d %d %d %d %d %d %d %d | %d %d | B %d %d %d %d | %d %d %d\n",
             ai->asvId, aj->asvId, ai->gaId, aj->gaId,
             ai->graphId, ai->ccId,
             aj->graphId, aj->ccId, st,
             ai->numExons, aj->numExons,
             es->numExonExactMatch,
             es->numExonExtendedMatch,
             es->numExonPartialMatch,
             es->numExonWeakMatch,
             es->numExonNoMatch,
             et->numExonExactMatch,
             et->numExonExtendedMatch,
             et->numExonPartialMatch,
             et->numExonWeakMatch,
             et->numExonNoMatch,
             compatibleXi,
             compatibleXj,
             es->numIntronExactMatch,
             es->numIntronPartialMatch,
             es->numIntronWeakMatch,
             es->numIntronNoMatch,
             et->numIntronExactMatch,
             et->numIntronPartialMatch,
             et->numIntronWeakMatch,
             et->numIntronNoMatch,
             compatibleXi,
             compatibleXj,
             es->numBpTP,
             es->numBpTN,
             es->numBpFP,
             es->numBpFN,
             compatibleBi,
             compatibleBj,
             compatibleBn);

    return 1;
}


static void exon_stats(ASV_t *ai, ASV_t *aj, int *exactX_out, int *extendedX_out, int *partialX_out, int *weakX_out, int *nomatchX_out, int Vm, int Vi)
{
    int i, j;
    int exactX, extendedX, partialX, weakX, nomatchX;
    int from_V, to_V;

    int mode = (ai->type==TXPT_TYPE && aj->type==TXPT_TYPE) ? TXPT_MODE : GENE_MODE;

    exactX = extendedX = partialX = weakX = nomatchX = 0;

    for (i=0; i<ai->numExons; i++) {
       Exon_t *xi = ai->exons+i;

       int isExact = 0;
       int isPartial = 0;
       int isExtended = 0;
       int isWeak = 0;


       for (j=0; j<aj->numExons; j++) {
          Exon_t *xj = aj->exons+j;
          int from_V, to_V;

          if (mode==TXPT_MODE) {
              if (xj->to<xi->from) continue;
              if (xj->from>xi->to) break;
          }
          
          from_V = (!i || !j) ? Vm : Vi;
          to_V = ((i==ai->numExons-1) || (j==aj->numExons-1)) ? Vm : Vi;

          if (abs(xi->from - xj->from)<=from_V && abs(xi->to - xj->to)<=to_V) {
              isExact = 1;
          } else if (i==ai->numExons-1 && abs(xi->from - xj->from)<=from_V) {
              isExtended = 1;
          } else if (i==0 && abs(xi->to - xj->to)<=to_V) {
              isExtended = 1;
          } else if (abs(xi->from - xj->from)<=from_V || abs(xi->to - xj->to)<=to_V) {
              isPartial = 1;
          } else {
              int ovl = overlaps(xi->from-V, xi->to+V, xj->from-V, xj->to+V);
              if (mode==TXPT_MODE) assert(ovl);
              if (ovl) isWeak = 1;
          }
       }
       if (isExact) exactX++;
       else if (isExtended) extendedX++;
       else if (isPartial) partialX++;
       else if (isWeak) weakX++;
       else nomatchX++;
    }

    *exactX_out         = exactX;
    *extendedX_out      = extendedX;
    *partialX_out       = partialX;
    *weakX_out          = weakX;
    *nomatchX_out       = nomatchX;

}


static void intron_stats(ASV_t *ai, ASV_t *aj, int *exactI_out, int *partialI_out, int *weakI_out, int *nomatchI_out, int Vm, int Vi)
{
    int i, j;
    int exactI, partialI, weakI, nomatchI;
    int from_V, to_V;

#ifdef INFO
    int exactFrom, exactTo, partialFrom, partialTo, weakFrom, weakTo;
#endif

    int mode = (ai->type==TXPT_TYPE && aj->type==TXPT_TYPE) ? TXPT_MODE : GENE_MODE;

    exactI = partialI = weakI = nomatchI = 0;

    for (i=0; i<ai->numIntrons; i++) {
       Intron_t *ei  = ai->introns+i;

       int isExact = 0;
       int isPartial = 0;
       int isWeak = 0;

       for (j=0; j<aj->numIntrons; j++) {
          Intron_t *ej  = aj->introns+j;
          int from_V, to_V;

          from_V = (!i || !j) ? Vm : Vi;
          to_V = ((i==ai->numExons-1) || (j==aj->numExons-1)) ? Vm : Vi;

          if (mode==TXPT_MODE) {
              if (ej->to<ei->from) continue;  /* intron j: ----  intron i: ---- (move j) */
              if (ej->from>ei->to) break;     /* intron i: ----  intron j: ---- (stop j) */
          }

          if (abs(ei->from - ej->from)<=from_V && abs(ei->to - ej->to)<=to_V) {
              isExact = 1;
#ifdef INFO
              exactFrom = ej->from; exactTo = ej->to; 
#endif
          } else if (abs(ei->from - ej->from)<=from_V || abs(ei->to - ej->to)<=to_V) {
              isPartial = 1;
#ifdef INFO
              partialFrom = ej->from; partialTo = ej->to;
#endif
          } else if (overlaps(ei->from-V, ei->to+V, ej->from-V, ej->to+V)) {
              isWeak = 1;
#ifdef INFO
              weakFrom = ej->from; weakTo = ej->to;
#endif
          }
       }

       if (isExact) exactI++;
       else if (isPartial) partialI++;
       else if (isWeak) weakI++;
       else nomatchI++;

#ifdef INFO
       if (isExact) printf("Intron %s %s %d %d %d %d Exact\n",
                           ai->asvId, aj->asvId, ei->from, ei->to, exactFrom, exactTo);
       else if (isPartial) printf("Intron %s %s %d %d %d %d Partial\n",
                           ai->asvId, aj->asvId, ei->from, ei->to, partialFrom, partialTo);
       else if (isWeak) printf("Intron %s %s %d %d %d %d Weak\n",
                           ai->asvId, aj->asvId, ei->from, ei->to, weakFrom, weakTo);
       else printf("Intron %s %s %d %d - - Nomatch\n",
                           ai->asvId, aj->asvId, ei->from, ei->to);
#endif

    }

   *exactI_out          = exactI;
   *partialI_out        = partialI;
   *weakI_out           = weakI;
   *nomatchI_out        = nomatchI;
}


static int compatible(ASV_t *ai, ASV_t *aj, int Vi, int Vm, int mode)
{
    if (mode == STRICT) 
       return compatible_STRICT(ai,aj,Vi,Vm);
    else if (mode == WEAK) 
       return compatible_WEAK(ai,aj,Vi,Vm);
    else {
       char errmsg[100];
       sprintf(errmsg, "%d", mode); 
       fatalf("Unrecognized mode (%s).\n", errmsg);
    }
}


static int compatible_WEAK(ASV_t *ai, ASV_t *aj, int Vi, int Vm) 
{
    int i, j;
    int starti, startj, endi, endj; 
    int from_V, to_V;
    int left, right;
    Intron_t *ei, *ej;

    if (ai->type!=TXPT_TYPE || aj->type!=TXPT_TYPE) return -1;

    if ((ai->exons[0].from>=aj->exons[aj->numExons-1].to) ||
        (aj->exons[0].from>=ai->exons[ai->numExons-1].to))
        return -1;

    /* if ai or aj has only one exon and no exon overlap, then incompatible */ 
    if ((!ai->numIntrons || !aj->numIntrons) && !exon_overlap(ai, aj))
        return -1;

    i = j = 0;
    ei = ai->introns +i;
    ej = aj->introns +j;

    starti = startj = endi = endj = -1;

    while (i<ai->numIntrons && j<aj->numIntrons) {

       if (abs(ei->from - ej->from)<=Vi && abs(ei->to - ej->to)<=Vi) {
          if (starti<0) { starti = i; startj = j; } 
          endi = i; endj = j;

          i++; j++;
          ei = ai->introns+i;
          ej = aj->introns+j;
       } else if (!j && ei->to<ej->from-Vi) {
          i++;
          ei = ai->introns+i;
       } else if (!i && ej->to<ei->from-Vi) {
          j++;
          ej = aj->introns+j;
       } else
          return -1;
    } 

    if (starti>=0) {
       if (starti && startj)
          fprintf(stderr, "Warning: Ambiguity - very short exon + intron at start.(%s,%s)\n",
                          ai->asvId, aj->asvId);
       assert(!starti || !startj);
       assert(endi==ai->numIntrons-1 || endj==aj->numIntrons-1);
    } /* else ai->numExons ==1 or aj->numExons==1 and their exons overlap*/     

    if (!ai->numIntrons || !aj->numIntrons) {
       if (abs(ai->exons[0].from-aj->exons[0].from)<=Vm) 
          left = 3;
       else if (ai->exons[0].from<aj->exons[0].from)
          left = 1;
       else 
          left = 2;

       if (abs(ai->exons[ai->numExons-1].to-aj->exons[aj->numExons-1].to)<=Vm) 
          right = 3;
       else if (aj->exons[aj->numExons-1].to<ai->exons[ai->numExons-1].to)
          right = 1;
       else 
          right = 2;

    } else {

       if (abs(ai->introns[0].from-aj->introns[0].from)<=Vi)
          left = 3;
       else if (ai->introns[0].from<aj->introns[0].from)
          left = 1;
       else
          left = 2;

       if (abs(ai->introns[ai->numIntrons-1].to-aj->introns[aj->numIntrons-1].to)<=Vi)
          right = 3;
       else if (aj->introns[aj->numIntrons-1].to<ai->introns[ai->numIntrons-1].to)
          right = 1;
       else
          right = 2;
    }

    return 10*left+right;
}


static int compatible_STRICT(ASV_t *ai, ASV_t *aj, int Vi, int Vm)
{
    int   i, j;
    int   starti, startj, endi, endj;
    int   left, right;

    if (ai->type!=TXPT_TYPE || aj->type!=TXPT_TYPE) return -1;

    if ((ai->exons[0].from>=aj->exons[aj->numExons-1].to) ||
        (aj->exons[0].from>=ai->exons[ai->numExons-1].to))
        return -1;

    starti = startj = endi = endj = -1;

    i = j = 0;
    if (ai->numExons>1 && aj->numExons>1) {
       int from_V, to_V; 

       while (i<ai->numExons && j<aj->numExons) {
          from_V = (!i || !j) ? Vm : Vi;
          to_V = ((i==ai->numExons-1) || (j==aj->numExons-1)) ? Vm : Vi;

          if ((i==ai->numExons-1) && (ai->exons[i].from<=aj->exons[j].from+from_V)) {
              if (starti<0) {
                  starti = i; startj = j;
              }
              endi = i; endj = j;
              i++; j++;
          } else if ((j==aj->numExons-1) && (aj->exons[j].from<=ai->exons[i].from+from_V)) {
              if (starti<0) {
                  starti = i; startj = j;
              }
              endi = i; endj = j;
              i++; j++;
          } else if ((i<ai->numExons-1) && (j<aj->numExons-1) &&
              abs(ai->exons[i].to - aj->exons[j].to)<=to_V &&
              abs(ai->exons[i+1].from - aj->exons[j+1].from)<=from_V) {
              if (starti<0) {
                  starti = i; startj = j;
              }
              endi = i+1; endj = j+1;
              i++; j++;
          } else if (!j && ai->exons[i].to<aj->exons[j].from-from_V) {
              i++;
          } else if (!i && aj->exons[j].to<ai->exons[i].from-from_V) {
              j++;
          } else {
             return -1;
          }
       }
       if (starti<0) return -1;

       assert(!starti || !startj);
       assert(endi==ai->numExons-1 || endj==aj->numExons-1);

       from_V = (!i || !j) ? Vm : Vi;
       to_V = ((i==ai->numExons-1) || (j==aj->numExons-1)) ? Vm : Vi;

       if (!starti && startj && ai->exons[starti].from<aj->exons[startj].from-from_V)
            return -1;
       if (starti && !startj && aj->exons[startj].from<ai->exons[starti].from-from_V)
            return -1;
       if (endi==ai->numExons-1 && endj<aj->numExons-1 &&
           ai->exons[endi].to>aj->exons[endj].to+to_V)
            return -1;
       if (endi<ai->numExons-1 && endj==aj->numExons-1 &&
           aj->exons[endj].to>ai->exons[endi].to+to_V)
            return -1;

    } else if (ai->numExons==1 && aj->numExons>1) {
       int from_V, to_V;

       for (j=0; j<aj->numExons; j++) {

          from_V = to_V = Vm;

          if ((!j && (ai->exons[0].to<=aj->exons[j].to+to_V)) ||
              (j==aj->numExons-1 && ai->exons[0].from>=aj->exons[j].from-from_V) ||
              (j && j!=aj->numExons-1 &&
               (ai->exons[0].from>=aj->exons[j].from-from_V &&
                ai->exons[0].to<=aj->exons[j].to+to_V))) {
              starti = endi = 0;
              startj = endj = j;
              break;
           }
       }
       if (starti<0) return -1;

    } else if (ai->numExons>1 && aj->numExons==1) {
       int from_V, to_V;

       for (i=0; i<ai->numExons; i++) {

          from_V = to_V = Vm;

          if ((!i && (aj->exons[0].to<=ai->exons[i].to+to_V)) ||
              (i==ai->numExons-1 && (aj->exons[0].from>=ai->exons[i].from-from_V)) ||
              (i && i!=ai->numExons-1 &&
               (aj->exons[0].from>=ai->exons[i].from-from_V &&
                aj->exons[0].to<=ai->exons[i].to+to_V))) {
              starti = endi = i;
              startj = endj = 0;
              break;
          }
       }
       if (starti<0) return -1;

    } /* (ai->numExons==1 && aj->numExons==1) is automatic */


    if (abs(ai->exons[0].from-aj->exons[0].from)<=Vm)
        left = 3;
    else if (ai->exons[0].from<aj->exons[0].from)
        left = 1;
    else
        left = 2;

    if (abs(ai->exons[ai->numExons-1].to-aj->exons[aj->numExons-1].to)<=Vm)
        right = 3;
    else if (aj->exons[aj->numExons-1].to<ai->exons[ai->numExons-1].to)
        right = 1;
    else
        right = 2;

    return 10*left+right;
}
