#ifndef H_COMPARE
#define H_COMPARE

#define EXON_EXACT_MATCH        10
#define EXON_EXTENDED_MATCH     11
#define EXON_PARTIAL_MATCH      12
#define EXON_WEAK_MATCH         13
#define EXON_NO_MATCH           14

#define INTRON_EXACT_MATCH      20
#define INTRON_PARTIAL_MATCH    21
#define INTRON_WEAK_MATCH       22
#define INTRON_NO_MATCH         23

#define COMPATIBLE_12           1*10+2
#define COMPATIBLE_21           2*10+1
#define COMPATIBLE_11           1*10+1
#define COMPATIBLE_22           2*10+2
#define COMPATIBLE_31           3*10+1
#define COMPATIBLE_32           3*10+2
#define COMPATIBLE_33           2*10+2
#define COMPATIBLE_13           1*10+3
#define COMPATIBLE_23           2*10+3
#define INCOMPATIBLE            -1

#define V                       10

#define DEFAULT_ALLOC_SMALL     5
#define DEFAULT_ALLOC_LARGE     100000

#define TXPT_MODE		10
#define GENE_MODE		11

#define TXPT_TYPE               't'
#define GENE_TYPE               'g'

#define STRICT                  10 
#define WEAK                    11

#if 0
#define max(x,y)   ((x)>=(y)?(x):(y))
#define min(x,y)   ((x)<=(y)?(x):(y))
#endif

typedef struct exon {
   long int nodeId;
   int      from, to;
} Exon_t;

typedef struct intron {
   long int edgeId;
   int      from, to;
} Intron_t;


typedef struct asv {
   char       asvId[500];
   int        gaId;
   int        graphId;
   int        ccId;
   int        strand;
   int        relori;
   int        b, e;
   int        numExons;
   int        maxExons;
   Exon_t    *exons;
   int        numIntrons;
   int        maxIntrons;
   Intron_t  *introns;
   char       type;
} ASV_t;


typedef struct exonstats {
   int       numExonExactMatch;
   int       numExonExtendedMatch;
   int       numExonPartialMatch;
   int       numExonWeakMatch;
   int       numExonNoMatch;

   int       numBpTP;
   int       numBpFP;
   int       numBpTN;
   int       numBpFN;

   int       numIntronExactMatch;
   int       numIntronPartialMatch;
   int       numIntronWeakMatch;
   int       numIntronNoMatch;
} CompareStats_t;


int overlaps(int,int,int,int);
int structure_compare(ASV_t *ai,
                      ASV_t *aj,
                      int *startI,
                      int *endI,
                      int *OH3,
                      int *OH5);
int feature_compare(ASV_t *ai,
                    ASV_t *aj,
                    CompareStats_t *es,
                    CompareStats_t *et,
                    int vi,
                    int vm,
                    int mode);

#endif
